<?php
/**
 * @file
 * Contains
 */

namespace Drupal\Tests\tranc\Unit;

use Drupal\Core\Datetime\DateFormatterInterface;
use Drupal\Core\Extension\Extension;
use Drupal\Core\Extension\ThemeHandlerInterface;
use Drupal\Core\Render\RendererInterface;
use Drupal\Core\Routing\UrlGeneratorInterface;
use Drupal\Core\Template\TwigExtension;
use Drupal\Core\Theme\ThemeManagerInterface;
use Drupal\Tests\UnitTestCase;
use Drupal\tranc\TrancTwigExtension;
use Twig\Environment;
use Twig\Loader\ArrayLoader;
use Twig\Source;

class TrancNodeVisitorTest extends UnitTestCase {

  /**
   * @var \Drupal\tranc\TrancTwigExtension
   */
  protected $trancTwigExtension;

  /**
   * @var \Drupal\Core\Template\TwigExtension
   */
  protected $coreTwigExtension;

  protected function setUp() {
    parent::setUp();
    $themeHandler = $this->prophesize(ThemeHandlerInterface::class);
    $themeHandler->getDefault()->willReturn('default');
    $themeHandler->getTheme('default')->willReturn(new Extension('', '', ''));
    $this->trancTwigExtension = new TrancTwigExtension($themeHandler->reveal());
    $renderer = $this->prophesize(RendererInterface::class)->reveal();
    $urlGenerator = $this->prophesize(UrlGeneratorInterface::class)->reveal();
    $themeManager = $this->prophesize(ThemeManagerInterface::class)->reveal();
    $dateFormatter = $this->prophesize(DateFormatterInterface::class)->reveal();
    $this->coreTwigExtension = new TwigExtension($renderer, $urlGenerator, $themeManager, $dateFormatter);
  }

  /**
   * @dataProvider providerTranc
   */
  public function testTrancNodeVisitor($string, $expected, $fire = TRUE) {
    // realpath() matches the new Extension('', '', '') in ::setUp() and this
    // makes TrancNodeVisitor fire always. On the other hand, realpath() will
    // never start with two slashes.
    $templateName = $fire ? realpath('') : '//';
    $loader = new ArrayLoader([$templateName => '']);
    $twig = new Environment($loader);
    $twig->addExtension($this->coreTwigExtension);
    $twig->addExtension($this->trancTwigExtension);
    $stream = $twig->tokenize(new Source($string, $templateName));
    $nodes = $twig->parse($stream);
    $this->assertContains($expected, $twig->compile($nodes));
  }

  public function providerTranc() {
    return [
      ['{% trans %}x{% endtrans %}', '"langcode" => "[content]"'],
      ["{% trans with {'context': 'foo'} %}x{% endtrans %}", '"langcode" => "[content]"'],
      ["{% trans with {'context': 'foo', 'langcode': 'bar'} %}x{% endtrans %}", '"langcode" => "bar"'],
      ['{{ foo|t }}', 'renderVar(tc('],
      ['{{ foo|t }}', 'renderVar(t(', FALSE],
    ];
  }

}
